@@ -1,42 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtDBus module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qdbusabstractinterface.h"
 #include "qdbusabstractinterface_p.h"
@@ -58,6 +22,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 namespace {
 // ### Qt6: change to a regular QEvent (customEvent)
 // We need to use a QMetaCallEvent here because we can't override customEvent() in
@@ -154,18 +120,17 @@ bool QDBusAbstractInterfacePrivate::property(const QMetaProperty &mp, void *retu
     if (!isValid || !canMakeCalls())   // can't make calls
         return false;
 
-    const int type = mp.userType();
+    QMetaType type = mp.metaType();
     // is this metatype registered?
     const char *expectedSignature = "";
-    if (int(mp.type()) != QMetaType::QVariant) {
+    if (type.id() != QMetaType::QVariant) {
         expectedSignature = QDBusMetaType::typeToSignature(type);
-        if (expectedSignature == 0) {
+        if (expectedSignature == nullptr) {
             qWarning("QDBusAbstractInterface: type %s must be registered with Qt D-Bus before it can be "
                      "used to read property %s.%s",
                      mp.typeName(), qPrintable(interface), mp.name());
-            lastError = QDBusError(QDBusError::Failed,
-                                   QLatin1String("Unregistered type %1 cannot be handled")
-                                   .arg(QLatin1String(mp.typeName())));
+            lastError = QDBusError(QDBusError::Failed, "Unregistered type %1 cannot be handled"_L1
+                                   .arg(QLatin1StringView(mp.typeName())));
             return false;
         }
     }
@@ -182,53 +147,53 @@ bool QDBusAbstractInterfacePrivate::property(const QMetaProperty &mp, void *retu
         lastError = QDBusError(reply);
         return false;
     }
-    if (reply.signature() != QLatin1String("v")) {
-        QString errmsg = QLatin1String("Invalid signature `%1' in return from call to "
-                                       DBUS_INTERFACE_PROPERTIES);
+    if (reply.signature() != "v"_L1) {
+        QString errmsg =
+                "Invalid signature '%1' in return from call to " DBUS_INTERFACE_PROPERTIES ""_L1;
         lastError = QDBusError(QDBusError::InvalidSignature, std::move(errmsg).arg(reply.signature()));
         return false;
     }
 
     QByteArray foundSignature;
-    const char *foundType = 0;
+    const char *foundType = nullptr;
     QVariant value = qvariant_cast<QDBusVariant>(reply.arguments().at(0)).variant();
 
-    if (value.userType() == type || type == QMetaType::QVariant
+    if (value.metaType() == type || type.id() == QMetaType::QVariant
         || (expectedSignature[0] == 'v' && expectedSignature[1] == '\0')) {
         // simple match
-        if (type == QMetaType::QVariant) {
+        if (type.id() == QMetaType::QVariant) {
             *reinterpret_cast<QVariant*>(returnValuePtr) = value;
         } else {
-            QMetaType::destruct(type, returnValuePtr);
-            QMetaType::construct(type, returnValuePtr, value.constData());
+            QMetaType(type).destruct(returnValuePtr);
+            QMetaType(type).construct(returnValuePtr, value.constData());
         }
         return true;
     }
 
-    if (value.userType() == qMetaTypeId<QDBusArgument>()) {
+    if (value.metaType() == QMetaType::fromType<QDBusArgument>()) {
         QDBusArgument arg = qvariant_cast<QDBusArgument>(value);
 
         foundType = "user type";
         foundSignature = arg.currentSignature().toLatin1();
         if (foundSignature == expectedSignature) {
             // signatures match, we can demarshall
-            return QDBusMetaType::demarshall(arg, type, returnValuePtr);
+            return QDBusMetaType::demarshall(arg, QMetaType(type), returnValuePtr);
         }
     } else {
         foundType = value.typeName();
-        foundSignature = QDBusMetaType::typeToSignature(value.userType());
+        foundSignature = QDBusMetaType::typeToSignature(value.metaType());
     }
 
     // there was an error...
-    const auto errmsg = QLatin1String("Unexpected `%1' (%2) when retrieving property `%3.%4' "
-                                      "(expected type `%5' (%6))");
+    const auto errmsg = "Unexpected '%1' (%2) when retrieving property '%3.%4' "
+                        "(expected type '%5' (%6))"_L1;
     lastError = QDBusError(QDBusError::InvalidSignature,
-                           errmsg.arg(QLatin1String(foundType),
-                                      QLatin1String(foundSignature),
+                           errmsg.arg(QLatin1StringView(foundType),
+                                      QLatin1StringView(foundSignature),
                                       interface,
-                                      QLatin1String(mp.name()),
-                                      QLatin1String(mp.typeName()),
-                                      QLatin1String(expectedSignature)));
+                                      QLatin1StringView(mp.name()),
+                                      QLatin1StringView(mp.typeName()),
+                                      QLatin1StringView(expectedSignature)));
     return false;
 }
 
@@ -281,10 +246,10 @@ int QDBusAbstractInterfaceBase::qt_metacall(QMetaObject::Call _c, int _id, void
 
         if (_c == QMetaObject::WriteProperty) {
             QVariant value;
-            if (mp.userType() == qMetaTypeId<QDBusVariant>())
+            if (mp.metaType() == QMetaType::fromType<QDBusVariant>())
                 value = reinterpret_cast<const QDBusVariant*>(_a[0])->variant();
             else
-                value = QVariant(mp.userType(), _a[0]);
+                value = QVariant(mp.metaType(), _a[0]);
             status = d_func()->setProperty(mp, value) ? 1 : 0;
         } else {
             bool readStatus = d_func()->property(mp, _a[0]);
@@ -460,7 +425,7 @@ QDBusMessage QDBusAbstractInterface::callWithArgumentList(QDBus::CallMode mode,
 
     QString m = method;
     // split out the signature from the method
-    int pos = method.indexOf(QLatin1Char('.'));
+    int pos = method.indexOf(u'.');
     if (pos != -1)
         m.truncate(pos);
 
@@ -597,7 +562,7 @@ bool QDBusAbstractInterface::callWithCallback(const QString &method,
                                               QObject *receiver,
                                               const char *slot)
 {
-    return callWithCallback(method, args, receiver, slot, 0);
+    return callWithCallback(method, args, receiver, slot, nullptr);
 }
 
 /*!
@@ -696,7 +661,7 @@ void QDBusAbstractInterface::internalPropSet(const char *propname, const QVarian
 */
 
 /*!
-    \fn QDBusAbstractInterface::call(const QString &message, Args&&...args)
+    \fn template <typename...Args> QDBusMessage QDBusAbstractInterface::call(const QString &method, Args&&...args)
 
     Calls the method \a method on this interface and passes \a args to the method.
     All \a args must be convertible to QVariant.
@@ -718,34 +683,13 @@ void QDBusAbstractInterface::internalPropSet(const char *propname, const QVarian
     \sa callWithArgumentList()
 */
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-/*!
-    \internal
-
-    This function exists for binary compatibility with Qt versions < 5.14.
-    Programs recompiled against Qt 5.14 will use the variadic template function
-    instead.
-*/
-QDBusMessage QDBusAbstractInterface::call(const QString &method, const QVariant &arg1,
-                                          const QVariant &arg2,
-                                          const QVariant &arg3,
-                                          const QVariant &arg4,
-                                          const QVariant &arg5,
-                                          const QVariant &arg6,
-                                          const QVariant &arg7,
-                                          const QVariant &arg8)
-{
-    return call(QDBus::AutoDetect, method, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
-}
-#endif
-
 /*!
     \fn QDBusAbstractInterface::call(QDBus::CallMode mode, const QString &message)
     \internal
 */
 
 /*!
-    \fn QDBusAbstractInterface::call(QDBus::CallMode mode, const QString &message, Args&&...args)
+    \fn template <typename...Args> QDBusMessage QDBusAbstractInterface::call(QDBus::CallMode mode, const QString &method, Args&&...args)
 
     \overload
 
@@ -768,66 +712,13 @@ QDBusMessage QDBusAbstractInterface::call(const QString &method, const QVariant
     \sa callWithArgumentList()
 */
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-/*!
-    \internal
-
-    This function exists for binary compatibility with Qt versions < 5.14.
-    Programs recompiled against Qt 5.14 will use the variadic template function
-    instead.
-*/
-QDBusMessage QDBusAbstractInterface::call(QDBus::CallMode mode, const QString &method,
-                                          const QVariant &arg1,
-                                          const QVariant &arg2,
-                                          const QVariant &arg3,
-                                          const QVariant &arg4,
-                                          const QVariant &arg5,
-                                          const QVariant &arg6,
-                                          const QVariant &arg7,
-                                          const QVariant &arg8)
-{
-    QList<QVariant> argList;
-    int count = 0 + arg1.isValid() + arg2.isValid() + arg3.isValid() + arg4.isValid() +
-                arg5.isValid() + arg6.isValid() + arg7.isValid() + arg8.isValid();
-
-    switch (count) {
-    case 8:
-        argList.prepend(arg8);
-        Q_FALLTHROUGH();
-    case 7:
-        argList.prepend(arg7);
-        Q_FALLTHROUGH();
-    case 6:
-        argList.prepend(arg6);
-        Q_FALLTHROUGH();
-    case 5:
-        argList.prepend(arg5);
-        Q_FALLTHROUGH();
-    case 4:
-        argList.prepend(arg4);
-        Q_FALLTHROUGH();
-    case 3:
-        argList.prepend(arg3);
-        Q_FALLTHROUGH();
-    case 2:
-        argList.prepend(arg2);
-        Q_FALLTHROUGH();
-    case 1:
-        argList.prepend(arg1);
-        break;
-    }
-
-    return callWithArgumentList(mode, method, argList);
-}
-#endif // Qt 5
-
 /*!
     \fn QDBusAbstractInterface::asyncCall(const QString &message)
     \internal
 */
 
 /*!
-    \fn QDBusAbstractInterface::asyncCall(const QString &message, Args&&...args)
+    \fn template <typename...Args> QDBusPendingCall QDBusAbstractInterface::asyncCall(const QString &method, Args&&...args)
 
     Calls the method \a method on this interface and passes \a args to the method.
     All \a args must be convertible to QVariant.
@@ -849,57 +740,6 @@ QDBusMessage QDBusAbstractInterface::call(QDBus::CallMode mode, const QString &m
     \sa asyncCallWithArgumentList()
 */
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-/*!
-    \internal
-
-    This function exists for binary compatibility with Qt versions < 5.14.
-    Programs recompiled against Qt 5.14 will use the variadic template function
-    instead.
-*/
-QDBusPendingCall QDBusAbstractInterface::asyncCall(const QString &method, const QVariant &arg1,
-                                                   const QVariant &arg2,
-                                                   const QVariant &arg3,
-                                                   const QVariant &arg4,
-                                                   const QVariant &arg5,
-                                                   const QVariant &arg6,
-                                                   const QVariant &arg7,
-                                                   const QVariant &arg8)
-{
-    QList<QVariant> argList;
-    int count = 0 + arg1.isValid() + arg2.isValid() + arg3.isValid() + arg4.isValid() +
-                arg5.isValid() + arg6.isValid() + arg7.isValid() + arg8.isValid();
-
-    switch (count) {
-    case 8:
-        argList.prepend(arg8);
-        Q_FALLTHROUGH();
-    case 7:
-        argList.prepend(arg7);
-        Q_FALLTHROUGH();
-    case 6:
-        argList.prepend(arg6);
-        Q_FALLTHROUGH();
-    case 5:
-        argList.prepend(arg5);
-        Q_FALLTHROUGH();
-    case 4:
-        argList.prepend(arg4);
-        Q_FALLTHROUGH();
-    case 3:
-        argList.prepend(arg3);
-        Q_FALLTHROUGH();
-    case 2:
-        argList.prepend(arg2);
-        Q_FALLTHROUGH();
-    case 1:
-        argList.prepend(arg1);
-        break;
-    }
-
-    return asyncCallWithArgumentList(method, argList);
-}
-#endif // Qt 5
 
 /*!
     \internal
