@@ -1,37 +1,13 @@
 #!/usr/bin/env python3
-#############################################################################
-##
-## Copyright (C) 2021 The Qt Company Ltd.
-## Contact: https://www.qt.io/licensing/
-##
-## This file is part of the test suite of the Qt Toolkit.
-##
-## $QT_BEGIN_LICENSE:GPL-EXCEPT$
-## Commercial License Usage
-## Licensees holding valid commercial Qt licenses may use this file in
-## accordance with the commercial license agreement provided with the
-## Software or, alternatively, in accordance with the terms contained in
-## a written agreement between you and The Qt Company. For licensing terms
-## and conditions see https://www.qt.io/terms-conditions. For further
-## information use the contact form at https://www.qt.io/contact-us.
-##
-## GNU General Public License Usage
-## Alternatively, this file may be used under the terms of the GNU
-## General Public License version 3 as published by the Free Software
-## Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-## included in the packaging of this file. Please review the following
-## information to ensure the GNU General Public License requirements will
-## be met: https://www.gnu.org/licenses/gpl-3.0.html.
-##
-## $QT_END_LICENSE$
-##
-#############################################################################
+# Copyright (C) 2021 The Qt Company Ltd.
+# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 """Script to generate C++ code from CLDR data in QLocaleXML form
 
 See ``cldr2qlocalexml.py`` for how to generate the QLocaleXML data itself.
-Pass the output file from that as first parameter to this script; pass
-the ISO 639-3 data file as second parameter; pass the root of the qtbase
-check-out as third parameter.
+Pass the output file from that as first parameter to this script; pass the ISO
+639-3 data file as second parameter. You can optionally pass the root of the
+qtbase check-out as third parameter; it defaults to the root of the qtbase
+check-out containing this script.
 
 The ISO 639-3 data file can be downloaded from the SIL website:
 
@@ -44,7 +20,7 @@ from pathlib import Path
 from typing import Optional
 
 from qlocalexml import QLocaleXmlReader
-from localetools import unicode2hex, wrap_list, Error, Transcriber, SourceFileEditor
+from localetools import unicode2hex, wrap_list, Error, Transcriber, SourceFileEditor, qtbase_root
 from iso639_3 import LanguageCodeData
 
 class LocaleKeySorter:
@@ -130,7 +106,7 @@ class StringData:
         if len(self.data) > 0xffff:
             raise ValueError(f'Data is too big ({len(self.data)}) for quint16 index to its end!',
                              self.name)
-        fd.write(f"\nstatic const char16_t {self.name}[] = {{\n")
+        fd.write(f"\nstatic constexpr char16_t {self.name}[] = {{\n")
         fd.write(wrap_list(self.data))
         fd.write("\n};\n")
 
@@ -175,7 +151,7 @@ class LocaleDataWriter (LocaleSourceEditor):
         likely = sorted(likely, key=keyLikely)
 
         i = 0
-        self.writer.write('static const QLocaleId likely_subtags[] = {\n')
+        self.writer.write('static constexpr QLocaleId likely_subtags[] = {\n')
         for had, have, got, give in likely:
             i += 1
             self.writer.write('    {{ {:3d}, {:3d}, {:3d} }}'.format(*have))
@@ -185,7 +161,7 @@ class LocaleDataWriter (LocaleSourceEditor):
         self.writer.write('};\n\n')
 
     def localeIndex(self, indices):
-        self.writer.write('static const quint16 locale_index[] = {\n')
+        self.writer.write('static constexpr quint16 locale_index[] = {\n')
         for index, name in indices:
             self.writer.write(f'{index:6d}, // {name}\n')
         self.writer.write('     0 // trailing 0\n')
@@ -206,7 +182,7 @@ class LocaleDataWriter (LocaleSourceEditor):
         endonyms_data = StringData('endonyms_data')
 
         # Locale data
-        self.writer.write('static const QLocaleData locale_data[] = {\n')
+        self.writer.write('static constexpr QLocaleData locale_data[] = {\n')
         # Table headings: keep each label centred in its field, matching line_format:
         self.writer.write('   // '
                           # Width 6 + comma
@@ -356,7 +332,7 @@ class LocaleDataWriter (LocaleSourceEditor):
 
     @staticmethod
     def __writeNameData(out, book, form):
-        out(f'static const char {form}_name_list[] =\n')
+        out(f'static constexpr char {form}_name_list[] =\n')
         out('"Default\\0"\n')
         for key, value in book.items():
             if key == 0:
@@ -364,7 +340,7 @@ class LocaleDataWriter (LocaleSourceEditor):
             out(f'"{value[0]}\\0"\n')
         out(';\n\n')
 
-        out(f'static const quint16 {form}_name_index[] = {{\n')
+        out(f'static constexpr quint16 {form}_name_index[] = {{\n')
         out(f'     0, // Any{form.capitalize()}\n')
         index = 8
         for key, value in book.items():
@@ -377,7 +353,7 @@ class LocaleDataWriter (LocaleSourceEditor):
 
     @staticmethod
     def __writeCodeList(out, book, form, width):
-        out(f'static const unsigned char {form}_code_list[] =\n')
+        out(f'static constexpr unsigned char {form}_code_list[] =\n')
         for key, value in book.items():
             code = value[1]
             code += r'\0' * max(width - len(code), 0)
@@ -403,18 +379,8 @@ class LocaleDataWriter (LocaleSourceEditor):
 
         def q(val: Optional[str], size: int) -> str:
             """Quote the value and adjust the result for tabular view."""
-            chars = []
-            if val is not None:
-                for c in val:
-                    chars.append(f"'{c}'")
-                s = ', '.join(chars)
-                s = f'{{{s}}}'
-            else:
-                s = ''
-            if size == 0:
-                return f'{{{s}}}'
-            else:
-                return f'{{{s}}},'.ljust(size * 5 + 4)
+            s = '' if val is None else ', '.join(f"'{c}'" for c in val)
+            return f'{{{s}}}' if size == 0 else f'{{{s}}},'.ljust(size * 5 + 2)
 
         for key, value in languages.items():
             code = value[1]
@@ -447,7 +413,7 @@ class CalendarDataWriter (LocaleSourceEditor):
     def write(self, calendar, locales, names):
         months_data = StringData('months_data')
 
-        self.writer.write('static const QCalendarLocale locale_data[] = {\n')
+        self.writer.write('static constexpr QCalendarLocale locale_data[] = {\n')
         self.writer.write(
             '     //'
             # IDs, width 7 (6 + comma)
@@ -546,8 +512,8 @@ class LocaleHeaderWriter (SourceFileEditor):
 
 
 def main(out, err):
-    # map { CLDR name: Qt file name }
     calendars_map = {
+        # CLDR name: Qt file name fragment
         'gregorian': 'roman',
         'persian': 'jalali',
         'islamic': 'hijri',
@@ -562,7 +528,8 @@ def main(out, err):
                         metavar='input-file.xml')
     parser.add_argument('iso_path', help='path to the ISO 639-3 data file',
                         metavar='iso-639-3.tab')
-    parser.add_argument('qtbase_path', help='path to the root of the qtbase source tree')
+    parser.add_argument('qtbase_path', help='path to the root of the qtbase source tree',
+                        nargs='?', default=qtbase_root)
     parser.add_argument('--calendars', help='select calendars to emit data for',
                         nargs='+', metavar='CALENDAR',
                         choices=all_calendars, default=all_calendars)
